/* 
**	doff - prints a formatted date string offset by a specified time interval
**
**	File:    doff.c
**	Author:  John Fitzgibbon (fitz@jfitz.com http://www.jfitz.com)
**	Version: 1.0
**	Date:    11/08/1999
**
** -------------------------------------------------------------------------------------
** Copyright/License/Disclaimer:
** -----------------------------
** Copyright (c) 2006 by John Fitzgibbon (fitz@jfitz.com).
**
** Permission to use, copy, modify, and distribute this software for any purpose
** with or without fee is hereby granted, provided that the above copyright notice,
** this permission notice and the following disclaimer appear in all copies.
**
** THIS SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
** TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
** IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
** CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
** PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
** ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
** -------------------------------------------------------------------------------------
**
**  Command line format: doff [format_str [offset_str [exclude_format_str]]]
**  doff -h to display help.
**
**  Revision History
**  17/06/2006     Added BSD-style License
**  11/08/1999 1.0 Original Version 
*/

char helpstr[] = 
"doff - Prints a formatted date string offset by a specified time interval\n"
"syntax is: doff [format_str [offset_str [exclude_format_str]]]\n"
"       or: doff -h   (displays this help screen)\n"
"\n"
"format_str, (case sensitive), may include any of the following:\n"
"  yyyy - replaced by four digit year in output string\n"
"  yy - replaced by two digit year in output string\n"
"  mm - replaced by month (1 - 12) in output string\n"
"  dd - replaced by day (1 - 31) in output string\n"
"  hh - replaced by hour (0 - 23) in output string\n"
"  mi - replaced by minute (0 - 59) in output string\n"
"  ss - replaced by second (0 - 59) in output string (leap seconds = 59)\n"
"Other characters in format_str are returned as-is\n"
"\n"
"offset_str, (case sensitive), is specified as [+/-]n[smhd]\n"
"  \"n\" is the offset value\n"
"  offset qualifiers are: s = seconds, m = minutes, h = hours, d = days\n"
"\n"
"exclude_format_str has the same format as format str.\n"
"Patterns in exclude_format_str that would normally be translated are ignored.\n"
"For example, including \"ss\" in exclude_format_str means \"ss\" will appear\n"
"untranslated in the output string.\n"
"\n"
"Default offset value is zero, default offset qualifier is days\n"
"Bad offset values default to zero, bad offset qualifiers default to days\n"
"Note: -10xd will evaluate to \"10 days ago\", (because atoi(\"-10x\") = -10)\n"
"  but -x10d will evaluate to \"now\", (because atoi(\"-x10\") = 0)\n"
"\n"
"When both format_str and offset_str are omitted the output format is:\n"
"yyyymmddhhmiss\n";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

int main (int argc, char * argv[])
{
  time_t etime;
  struct tm * stime;
  char dstr[15];
  char * cp;


  if ((argc > 1) && !strncmp(argv[1],"-h",2))
  {
    puts(helpstr);
    exit(0);
  }
  time(&etime);
  if (argc > 2)
  {
    if ((cp = strstr(argv[2],"s")) != NULL) // offset seconds
    {
      *cp = '\0';
      etime += atoi(argv[2]);
    }
    else if ((cp = strstr(argv[2],"m")) != NULL) // offset minutes
    {
      *cp = '\0';
      etime += 60*atoi(argv[2]);
    }
    else if ((cp = strstr(argv[2],"h")) != NULL) //offset hours
    {
      *cp = '\0';
      etime += 60*60*atoi(argv[2]);
    }
    else if ((cp = strstr(argv[2],"d")) != NULL) // offset days
    {
      *cp = '\0';
      etime += 60*60*24*atoi(argv[2]);
    }
    else // offset type not specified - assume days
      etime += 60*60*24*atoi(argv[2]);
  }
  stime = localtime(&etime);
  sprintf(dstr,"%4.4d%2.2d%2.2d%2.2d%2.2d%2.2d",
    stime->tm_year+1900,
    stime->tm_mon+1,
    stime->tm_mday,
    stime->tm_hour,
    stime->tm_min,
    ((stime->tm_sec == 60 || stime->tm_sec == 61) ? 59 : stime->tm_sec)
  );
  if (argc > 1)
  {
    if ((argc <= 3) || (strstr(argv[3],"yyyy") == NULL))
      while ((cp = strstr(argv[1],"yyyy")) != NULL) // format 4-digit year
        strncpy(cp,dstr,4);
    if ((argc <= 3) || (strstr(argv[3],"yy") == NULL))
      while ((cp = strstr(argv[1],"yy")) != NULL) // format 2-digit year
        strncpy(cp,dstr+2,2);
    if ((argc <= 3) || (strstr(argv[3],"mm") == NULL))
      while ((cp = strstr(argv[1],"mm")) != NULL) // format month
        strncpy(cp,dstr+4,2);
    if ((argc <= 3) || (strstr(argv[3],"dd") == NULL))
      while ((cp = strstr(argv[1],"dd")) != NULL) // format day
        strncpy(cp,dstr+6,2);
    if ((argc <= 3) || (strstr(argv[3],"hh") == NULL))
      while ((cp = strstr(argv[1],"hh")) != NULL) // format hour (24 hour format)
        strncpy(cp,dstr+8,2);
    if ((argc <= 3) || (strstr(argv[3],"mi") == NULL))
      while ((cp = strstr(argv[1],"mi")) != NULL) // format minute
        strncpy(cp,dstr+10,2);
    if ((argc <= 3) || (strstr(argv[3],"ss") == NULL))
      while ((cp = strstr(argv[1],"ss")) != NULL) // format second
        strncpy(cp,dstr+12,2);
    puts(argv[1]);
  }
  else
    puts(dstr);

  return (0);
}

